@@ -33,7 +33,6 @@
 #endif
 
 #include "sox_i.h"
-#include "sgetopt.h"
 #include <assert.h>
 
 #define MAX_FRAME_LENGTH 8192
@@ -44,7 +43,7 @@ typedef struct {
     char *str;           /* Command-line argument to parse for this bend */
     size_t start;        /* Start bending when in_pos equals this */
     double cents;
-    size_t duration;     /* Number of samples to bend */
+    uint64_t duration;     /* Number of samples to bend */
   } *bends;
 
   unsigned frame_rate;
@@ -70,7 +69,8 @@ typedef struct {
 static int parse(sox_effect_t * effp, char **argv, sox_rate_t rate)
 {
   priv_t *p = (priv_t *) effp->priv;
-  size_t i, time = 0, delay;
+  size_t i;
+  uint64_t time = 0, delay;
   char const *next;
 
   for (i = 0; i < p->nbends; ++i) {
@@ -98,17 +98,20 @@ static int create(sox_effect_t * effp, int argc, char **argv)
   priv_t *p = (priv_t *) effp->priv;
   char const * opts = "f:o:";
   int c;
+  lsx_getopt_t optstate;
+  lsx_getopt_init(argc, argv, opts, NULL, lsx_getopt_flag_none, 1, &optstate);
 
   p->frame_rate = 25;
   p->ovsamp = 16;
-  while ((c = lsx_getopt(argc, argv, opts)) != -1) switch (c) {
-    GETOPT_NUMERIC('f', frame_rate, 10 , 80)
-    GETOPT_NUMERIC('o', ovsamp,  4 , 32)
-    default: lsx_fail("unknown option `-%c'", optopt); return lsx_usage(effp);
+  while ((c = lsx_getopt(&optstate)) != -1) switch (c) {
+    GETOPT_NUMERIC(optstate, 'f', frame_rate, 10 , 80)
+    GETOPT_NUMERIC(optstate, 'o', ovsamp,  4 , 32)
+    default: lsx_fail("unknown option `-%c'", optstate.opt); return lsx_usage(effp);
   }
-  argc -= lsx_optind, argv += lsx_optind;
+  argc -= optstate.ind, argv += optstate.ind;
 
-  p->bends = lsx_calloc(p->nbends = argc, sizeof(*p->bends));
+  p->nbends = argc;
+  p->bends = lsx_calloc(p->nbends, sizeof(*p->bends));
   return parse(effp, argv, 0.);     /* No rate yet; parse with dummy */
 }
 
