@@ -76,7 +76,7 @@ static int getopts(sox_effect_t * effp, int argc, char *argv[])
     NUMERIC_PARAMETER(feedback_gain,-95 , 95 )
     NUMERIC_PARAMETER(delay_gain   , 0  , 100)
     NUMERIC_PARAMETER(speed        , 0.1, 10 )
-    TEXTUAL_PARAMETER(wave_shape, lsx_wave_enum)
+    TEXTUAL_PARAMETER(wave_shape, lsx_get_wave_enum())
     NUMERIC_PARAMETER(channel_phase, 0  , 100)
     TEXTUAL_PARAMETER(interpolation, interp_enum)
   } while (0);
@@ -98,7 +98,7 @@ static int getopts(sox_effect_t * effp, int argc, char *argv[])
       p->feedback_gain,
       p->delay_gain,
       p->speed,
-      lsx_wave_enum[p->wave_shape].text,
+      lsx_get_wave_enum()[p->wave_shape].text,
       p->channel_phase,
       interp_enum[p->interpolation].text);
 
@@ -154,8 +154,8 @@ static int start(sox_effect_t * effp)
       f->delay_buf_length - 2.,
       3 * M_PI_2);  /* Start the sweep at minimum delay (for mono at least) */
 
-  lsx_debug("delay_buf_length=%lu lfo_length=%lu\n",
-      (unsigned long)f->delay_buf_length, (unsigned long)f->lfo_length);
+  lsx_debug("delay_buf_length=%" PRIuPTR " lfo_length=%" PRIuPTR "\n",
+      f->delay_buf_length, f->lfo_length);
 
   return SOX_SUCCESS;
 }
