@@ -44,6 +44,7 @@
   FORMAT(s3)
   FORMAT(s4)
   FORMAT(sf)
+  FORMAT(sln)
   FORMAT(smp)
   FORMAT(sounder)
   FORMAT(soundtool)
@@ -108,9 +109,7 @@
 #endif
 #if defined HAVE_SNDFILE && (defined STATIC_SNDFILE || !defined HAVE_LIBLTDL)
   FORMAT(sndfile)
-  #if defined HAVE_SNDFILE_1_0_12
   FORMAT(caf)
-  #endif
   FORMAT(fap)
   FORMAT(mat4)
   FORMAT(mat5)
