@@ -38,11 +38,12 @@ typedef struct {
 /*
  * Process options
  */
-static int getopts(sox_effect_t * effp, int n, char **argv)
+static int getopts(sox_effect_t * effp, int argc, char **argv)
 {
     priv_t * mixer = (priv_t *) effp->priv;
     double* pans = &mixer->sources[0][0];
     int i;
+  --argc, ++argv;
 
     for (i = 0;  i < 16;  i++)
         pans[i] = 0.0;
@@ -52,7 +53,7 @@ static int getopts(sox_effect_t * effp, int n, char **argv)
     /* Parse parameters.  Since we don't yet know the number of */
     /* input and output channels, we'll record the information for */
     /* later. */
-    if (n == 1) {
+    if (argc == 1) {
         if      (!strcmp(argv[0], "-l")) mixer->mix = 'l';
         else if (!strcmp(argv[0], "-r")) mixer->mix = 'r';
         else if (!strcmp(argv[0], "-f")) mixer->mix = 'f';
@@ -82,7 +83,7 @@ static int getopts(sox_effect_t * effp, int n, char **argv)
             mixer->num_pans = commas + 1;
         }
     }
-    else if (n == 0) {
+    else if (argc == 0) {
         mixer->mix = MIX_CENTER;
     }
     else
@@ -484,6 +485,19 @@ static int start(sox_effect_t * effp)
          }
      }
 
+     if (effp->in_signal.mult) {
+       double max_sum = 0;
+
+       for (j = 0; j < (int)effp->out_signal.channels; ++j) {
+         double sum = 0;
+         for (i = 0; i < (int)effp->in_signal.channels; ++i)
+           sum += fabs(mixer->sources[mixer->mix == MIX_CENTER? 0 : i][j]);
+         max_sum = max(max_sum, sum);
+       }
+       if (max_sum > 1)
+         *effp->in_signal.mult /= max_sum;
+     }
+
      if (effp->in_signal.channels != effp->out_signal.channels)
        return SOX_SUCCESS;
 
@@ -527,29 +541,13 @@ static int flow(sox_effect_t * effp, const sox_sample_t *ibuf, sox_sample_t *obu
     return (SOX_SUCCESS);
 }
 
-sox_effect_handler_t const * sox_mixer_effect_fn(void)
+sox_effect_handler_t const * lsx_mixer_effect_fn(void)
 {
   static sox_effect_handler_t handler = {
     "mixer",
     "[ -l | -r | -f | -b | -1 | -2 | -3 | -4 | n,n,n...,n ]",
-    SOX_EFF_MCHAN | SOX_EFF_CHAN,
+    SOX_EFF_MCHAN | SOX_EFF_CHAN | SOX_EFF_GAIN | SOX_EFF_DEPRECATED,
     getopts, start, flow, 0, 0, 0, sizeof(priv_t)
   };
   return &handler;
 }
-
-static int oops_getopts(sox_effect_t * effp, int argc, char * * argv UNUSED)
-{
-  char * args[] = {"1,1,-1,-1"};
-  return argc? lsx_usage(effp) : sox_mixer_effect_fn()->getopts(effp, (int)array_length(args), args);
-}
-
-sox_effect_handler_t const * sox_oops_effect_fn(void)
-{
-  static sox_effect_handler_t handler;
-  handler = *sox_mixer_effect_fn();
-  handler.name = "oops";
-  handler.usage = NULL;
-  handler.getopts = oops_getopts;
-  return &handler;
-}
