@@ -1,7 +1,8 @@
 /* MP3 support for SoX
  *
  * Uses libmad for MP3 decoding
- * and libmp3lame for MP3 encoding
+ * libmp3lame for MP3 encoding
+ * and libtwolame for MP2 encoding
  *
  * Written by Fabrizio Gennari <fabrizio.ge@tiscali.it>
  *
@@ -12,13 +13,15 @@
 #include "sox_i.h"
 #include <string.h>
 
-#if defined(HAVE_LAME_LAME_H) || defined(HAVE_LAME_H)
+#if defined(HAVE_LAME_LAME_H) || defined(HAVE_LAME_H) || defined(DL_LAME)
 #define HAVE_LAME 1
-#else
-#undef HAVE_LAME
 #endif
 
-#if defined(HAVE_MAD_H) || defined(HAVE_LAME)
+#if defined(HAVE_TWOLAME_H) || defined(DL_TWOLAME)
+  #define HAVE_TWOLAME 1
+#endif
+
+#if defined(HAVE_MAD_H) || defined(HAVE_LAME) || defined(HAVE_TWOLAME)
 
 #ifdef HAVE_MAD_H
 #include <mad.h>
@@ -28,6 +31,12 @@
 #include <lame/lame.h>
 #elif defined(HAVE_LAME_H)
 #include <lame.h>
+#elif defined(DL_LAME)
+typedef struct lame_global_struct lame_global_flags;
+typedef enum {
+  vbr_off=0,
+  vbr_default=4
+} vbr_mode;
 #endif
 
 #if defined(HAVE_ID3TAG) && (defined(HAVE_IO_H) || defined(HAVE_UNISTD_H))
@@ -45,6 +54,10 @@
   #define ID3_TAG_FLAG_FOOTERPRESENT 0x10
 #endif
 
+#ifdef HAVE_TWOLAME_H
+  #include <twolame.h>
+#endif
+
 #ifndef HAVE_LIBLTDL
   #undef DL_LAME
   #undef DL_MAD
@@ -56,8 +69,10 @@
  * Sidestep the issue by defining our own mad_timer_zero. This is needed because
  * mad_timer_zero is used in some of the mad.h macros.
  */
+#ifdef HAVE_MAD_H
 #define mad_timer_zero mad_timer_zero_stub
 static mad_timer_t const mad_timer_zero_stub = {0, 0};
+#endif
 
 #define MAXFRAMESIZE 2880
 #define ID3PADDING 128
@@ -74,6 +89,7 @@ static const char* const mad_library_names[] =
 {
 #ifdef DL_MAD
     "libmad",
+    "libmad-0",
     "cygmad-0",
 #endif
     NULL
@@ -108,6 +124,7 @@ static const char* const lame_library_names[] =
 {
 #ifdef DL_LAME
   "libmp3lame",
+  "libmp3lame-0",
   "lame-enc",
   "cygmp3lame-0",
 #endif
@@ -122,13 +139,6 @@ static const char* const lame_library_names[] =
   /* id3tag support is an optional component of LAME. Use if available. */
   #define LAME_FUNC_ID3       LSX_DLENTRY_STUB
 
-  /* Added in LAME 3.98: lame_get_lametag_frame. Use if available. */
-  #define LAME_FUNC_398       LSX_DLENTRY_STUB
-
-  /* Added in LAME 3.98.1 (optional): id3tag_set_pad, lame_get_id3v2_tag,
-     id3tag_set_fieldvalue. Use if available. */
-  #define LAME_FUNC_398_ID3   LSX_DLENTRY_STUB
-
 #else /* DL_LAME */
 
   /* Expected to be present in all builds of LAME. */
@@ -141,21 +151,6 @@ static const char* const lame_library_names[] =
     #define LAME_FUNC_ID3     LSX_DLENTRY_STUB
   #endif
 
-  /* Added in LAME 3.98: lame_get_lametag_frame. Use if available. */
-  #ifdef HAVE_LAME_398
-    #define LAME_FUNC_398     LSX_DLENTRY_STATIC
-  #else
-    #define LAME_FUNC_398     LSX_DLENTRY_STUB
-  #endif
-
-  /* Added in LAME 3.98.1 (optional): id3tag_set_pad, lame_get_id3v2_tag,
-     id3tag_set_fieldvalue. Use if available. */
-  #ifdef HAVE_LAME_398_ID3TAG
-    #define LAME_FUNC_398_ID3 LSX_DLENTRY_STATIC
-  #else
-    #define LAME_FUNC_398_ID3 LSX_DLENTRY_STUB
-  #endif
-
 #endif /* DL_LAME */
 
 #define LAME_FUNC_ENTRIES(f,x) \
@@ -178,6 +173,7 @@ static const char* const lame_library_names[] =
   LAME_FUNC(f,x, int, lame_encode_buffer_float, (lame_global_flags *, const float[], const float[], const int, unsigned char *, const int)) \
   LAME_FUNC(f,x, int, lame_encode_flush, (lame_global_flags *, unsigned char *, int)) \
   LAME_FUNC(f,x, int, lame_close, (lame_global_flags *)) \
+  LAME_FUNC(f,x, size_t, lame_get_lametag_frame, (const lame_global_flags *, unsigned char*, size_t)) \
   LAME_FUNC_ID3(f,x, void, id3tag_init, (lame_global_flags *)) \
   LAME_FUNC_ID3(f,x, void, id3tag_set_title, (lame_global_flags *, const char* title)) \
   LAME_FUNC_ID3(f,x, void, id3tag_set_artist, (lame_global_flags *, const char* artist)) \
@@ -186,10 +182,36 @@ static const char* const lame_library_names[] =
   LAME_FUNC_ID3(f,x, void, id3tag_set_comment, (lame_global_flags *, const char* comment)) \
   LAME_FUNC_ID3(f,x, int, id3tag_set_track, (lame_global_flags *, const char* track)) \
   LAME_FUNC_ID3(f,x, int, id3tag_set_genre, (lame_global_flags *, const char* genre)) \
-  LAME_FUNC_398(f,x, size_t, lame_get_lametag_frame, (const lame_global_flags *, unsigned char*, size_t)) \
-  LAME_FUNC_398_ID3(f,x, size_t, id3tag_set_pad, (lame_global_flags *, size_t)) \
-  LAME_FUNC_398_ID3(f,x, size_t, lame_get_id3v2_tag, (lame_global_flags *, unsigned char*, size_t)) \
-  LAME_FUNC_398_ID3(f,x, int, id3tag_set_fieldvalue, (lame_global_flags *, const char *))
+  LAME_FUNC_ID3(f,x, size_t, id3tag_set_pad, (lame_global_flags *, size_t)) \
+  LAME_FUNC_ID3(f,x, size_t, lame_get_id3v2_tag, (lame_global_flags *, unsigned char*, size_t)) \
+  LAME_FUNC_ID3(f,x, int, id3tag_set_fieldvalue, (lame_global_flags *, const char *))
+
+static const char* const twolame_library_names[] =
+{
+#ifdef DL_TWOLAME
+  "libtwolame",
+  "libtwolame-0",
+#endif
+  NULL
+};
+
+#ifdef DL_TWOLAME
+  #define TWOLAME_FUNC LSX_DLENTRY_DYNAMIC
+#else
+  #define TWOLAME_FUNC LSX_DLENTRY_STATIC
+#endif
+
+#define TWOLAME_FUNC_ENTRIES(f,x) \
+  TWOLAME_FUNC(f,x, twolame_options*, twolame_init, (void)) \
+  TWOLAME_FUNC(f,x, int, twolame_get_num_channels, (twolame_options*)) \
+  TWOLAME_FUNC(f,x, int, twolame_set_num_channels, (twolame_options*, int)) \
+  TWOLAME_FUNC(f,x, int, twolame_set_in_samplerate, (twolame_options *, int)) \
+  TWOLAME_FUNC(f,x, int, twolame_set_out_samplerate, (twolame_options *, int)) \
+  TWOLAME_FUNC(f,x, int, twolame_set_brate, (twolame_options *, int)) \
+  TWOLAME_FUNC(f,x, int, twolame_init_params, (twolame_options *)) \
+  TWOLAME_FUNC(f,x, int, twolame_encode_buffer_float32_interleaved, (twolame_options *, const float [], int, unsigned char *, int)) \
+  TWOLAME_FUNC(f,x, int, twolame_encode_flush, (twolame_options *, unsigned char *, int)) \
+  TWOLAME_FUNC(f,x, void, twolame_close, (twolame_options **))
 
 /* Private data */
 typedef struct mp3_priv_t {
@@ -206,16 +228,27 @@ typedef struct mp3_priv_t {
   LSX_DLENTRIES_TO_PTRS(MAD_FUNC_ENTRIES, mad_dl);
 #endif /*HAVE_MAD_H*/
 
-#ifdef HAVE_LAME
-  lame_global_flags *gfp;
+#if defined(HAVE_LAME) || defined(HAVE_TWOLAME)
   float *pcm_buffer;
   size_t pcm_buffer_size;
-  unsigned long num_samples;
+  char mp2;
+#endif
+
+#ifdef HAVE_LAME
+  lame_global_flags *gfp;
+  uint64_t num_samples;
   int vbr_tag;
   LSX_DLENTRIES_TO_PTRS(LAME_FUNC_ENTRIES, lame_dl);
-#endif /*HAVE_LAME*/
+#endif
+
+#ifdef HAVE_TWOLAME
+  twolame_options *opt;
+  LSX_DLENTRIES_TO_PTRS(TWOLAME_FUNC_ENTRIES, twolame_dl);
+#endif
 } priv_t;
 
+#ifdef HAVE_MAD_H
+
 /* This function merges the functions tagtype() and id3_tag_query()
    from MAD's libid3tag, so we don't have to link to it
    Returns 0 if the frame is not an ID3 tag, tag length if it is */
@@ -245,6 +278,8 @@ static int tagtype(const unsigned char *data, size_t length)
     return 0;
 }
 
+#endif /*HAVE_MAD_H*/
+
 #include "mp3-util.h"
 
 #ifdef HAVE_MAD_H
@@ -350,7 +385,7 @@ static int startread(sox_format_t * ft)
   if (ft->seekable) {
 #ifdef USING_ID3TAG
     read_comments(ft);
-    rewind(ft->fp);
+    rewind((FILE*)ft->fp);
     if (!ft->signal.length)
 #endif
       if (!ignore_length)
@@ -369,7 +404,7 @@ static int startread(sox_format_t * ft)
    * can be processed later.
    */
   ReadSize = lsx_readbuf(ft, p->mp3_buffer, p->mp3_buffer_size);
-  if (ReadSize != p->mp3_buffer_size && ferror(ft->fp))
+  if (ReadSize != p->mp3_buffer_size && ferror((FILE*)ft->fp))
     return SOX_EOF;
 
   p->mad_stream_buffer(&p->Stream, p->mp3_buffer, ReadSize);
@@ -429,7 +464,7 @@ static int startread(sox_format_t * ft)
   if (ignore_length)
     ft->signal.length = SOX_UNSPEC;
   else {
-    ft->signal.length = (size_t)(ft->signal.length * .001 * ft->signal.rate + .5);
+    ft->signal.length = (uint64_t)(ft->signal.length * .001 * ft->signal.rate + .5);
     ft->signal.length *= ft->signal.channels;  /* Keep separate from line above! */
   }
 
@@ -522,7 +557,111 @@ static int stopread(sox_format_t * ft)
   LSX_DLLIBRARY_CLOSE(p, mad_dl);
   return SOX_SUCCESS;
 }
-#else /*HAVE_MAD_H*/
+
+static int sox_mp3seek(sox_format_t * ft, uint64_t offset)
+{
+  priv_t   * p = (priv_t *) ft->priv;
+  size_t   initial_bitrate = p->Frame.header.bitrate;
+  size_t   tagsize = 0, consumed = 0;
+  sox_bool vbr = sox_false; /* Variable Bit Rate */
+  sox_bool depadded = sox_false;
+  uint64_t to_skip_samples = 0;
+
+  /* Reset all */
+  rewind((FILE*)ft->fp);
+  mad_timer_reset(&p->Timer);
+  p->FrameCount = 0;
+
+  /* They where opened in startread */
+  mad_synth_finish(&p->Synth);
+  p->mad_frame_finish(&p->Frame);
+  p->mad_stream_finish(&p->Stream);
+
+  p->mad_stream_init(&p->Stream);
+  p->mad_frame_init(&p->Frame);
+  p->mad_synth_init(&p->Synth);
+
+  offset /= ft->signal.channels;
+  to_skip_samples = offset;
+
+  while(sox_true) {  /* Read data from the MP3 file */
+    int read, padding = 0;
+    size_t leftover = p->Stream.bufend - p->Stream.next_frame;
+
+    memcpy(p->mp3_buffer, p->Stream.this_frame, leftover);
+    read = fread(p->mp3_buffer + leftover, (size_t) 1, p->mp3_buffer_size - leftover, (FILE*)ft->fp);
+    if (read <= 0) {
+      lsx_debug("seek failure. unexpected EOF (frames=%" PRIuPTR " leftover=%" PRIuPTR ")", p->FrameCount, leftover);
+      break;
+    }
+    for (; !depadded && padding < read && !p->mp3_buffer[padding]; ++padding);
+    depadded = sox_true;
+    p->mad_stream_buffer(&p->Stream, p->mp3_buffer + padding, leftover + read - padding);
+
+    while (sox_true) {  /* Decode frame headers */
+      static unsigned short samples;
+      p->Stream.error = MAD_ERROR_NONE;
+
+      /* Not an audio frame */
+      if (p->mad_header_decode(&p->Frame.header, &p->Stream) == -1) {
+        if (p->Stream.error == MAD_ERROR_BUFLEN)
+          break;  /* Normal behaviour; get some more data from the file */
+        if (!MAD_RECOVERABLE(p->Stream.error)) {
+          lsx_warn("unrecoverable MAD error");
+          break;
+        }
+        if (p->Stream.error == MAD_ERROR_LOSTSYNC) {
+          unsigned available = (p->Stream.bufend - p->Stream.this_frame);
+          tagsize = tagtype(p->Stream.this_frame, (size_t) available);
+          if (tagsize) {   /* It's some ID3 tags, so just skip */
+            if (tagsize >= available) {
+              fseeko((FILE*)ft->fp, (off_t)(tagsize - available), SEEK_CUR);
+              depadded = sox_false;
+            }
+            p->mad_stream_skip(&p->Stream, min(tagsize, available));
+          }
+          else lsx_warn("MAD lost sync");
+        }
+        else lsx_warn("recoverable MAD error");
+        continue;
+      }
+
+      consumed += p->Stream.next_frame - p->Stream.this_frame;
+      vbr      |= (p->Frame.header.bitrate != initial_bitrate);
+
+      samples = 32 * MAD_NSBSAMPLES(&p->Frame.header);
+
+      p->FrameCount++;
+      p->mad_timer_add(&p->Timer, p->Frame.header.duration);
+
+      if(to_skip_samples <= samples)
+      {
+        p->mad_frame_decode(&p->Frame,&p->Stream);
+        p->mad_synth_frame(&p->Synth, &p->Frame);
+        p->cursamp = to_skip_samples;
+        return SOX_SUCCESS;
+      }
+      else to_skip_samples -= samples;
+
+      /* If not VBR, we can extrapolate frame size */
+      if (p->FrameCount == 64 && !vbr) {
+        p->FrameCount = offset / samples;
+        to_skip_samples = offset % samples;
+
+        if (SOX_SUCCESS != lsx_seeki(ft, (off_t)(p->FrameCount * consumed / 64 + tagsize), SEEK_SET))
+          return SOX_EOF;
+
+        /* Reset Stream for refilling buffer */
+        p->mad_stream_finish(&p->Stream);
+        p->mad_stream_init(&p->Stream);
+        break;
+      }
+    }
+  };
+
+  return SOX_EOF;
+}
+#else /* !HAVE_MAD_H */
 static int startread(sox_format_t * ft)
 {
   lsx_fail_errno(ft,SOX_EOF,"SoX was compiled without MP3 decoding support");
@@ -530,6 +669,7 @@ static int startread(sox_format_t * ft)
 }
 #define sox_mp3read NULL
 #define stopread NULL
+#define sox_mp3seek NULL
 #endif /*HAVE_MAD_H*/
 
 #ifdef HAVE_LAME
@@ -563,29 +703,27 @@ static void msgf(const char* fmt, va_list va)
 /* These functions are considered optional. If they aren't present in the
    library, the stub versions defined here will be used instead. */
 
-static void id3tag_init_stub(lame_global_flags * gfp UNUSED)
+UNUSED static void id3tag_init_stub(lame_global_flags * gfp UNUSED)
   { return; }
-static void id3tag_set_title_stub(lame_global_flags * gfp UNUSED, const char* title UNUSED)
+UNUSED static void id3tag_set_title_stub(lame_global_flags * gfp UNUSED, const char* title UNUSED)
   { return; }
-static void id3tag_set_artist_stub(lame_global_flags * gfp UNUSED, const char* artist UNUSED)
+UNUSED static void id3tag_set_artist_stub(lame_global_flags * gfp UNUSED, const char* artist UNUSED)
   { return; }
-static void id3tag_set_album_stub(lame_global_flags * gfp UNUSED, const char* album UNUSED)
+UNUSED static void id3tag_set_album_stub(lame_global_flags * gfp UNUSED, const char* album UNUSED)
   { return; }
-static void id3tag_set_year_stub(lame_global_flags * gfp UNUSED, const char* year UNUSED)
+UNUSED static void id3tag_set_year_stub(lame_global_flags * gfp UNUSED, const char* year UNUSED)
   { return; }
-static void id3tag_set_comment_stub(lame_global_flags * gfp UNUSED, const char* comment UNUSED)
+UNUSED static void id3tag_set_comment_stub(lame_global_flags * gfp UNUSED, const char* comment UNUSED)
   { return; }
-static void id3tag_set_track_stub(lame_global_flags * gfp UNUSED, const char* track UNUSED)
+UNUSED static void id3tag_set_track_stub(lame_global_flags * gfp UNUSED, const char* track UNUSED)
   { return; }
-static int id3tag_set_genre_stub(lame_global_flags * gfp UNUSED, const char* genre UNUSED)
+UNUSED static int id3tag_set_genre_stub(lame_global_flags * gfp UNUSED, const char* genre UNUSED)
   { return 0; }
-static size_t lame_get_lametag_frame_stub(const lame_global_flags * gfp UNUSED, unsigned char * buffer UNUSED, size_t size UNUSED)
+UNUSED static size_t id3tag_set_pad_stub(lame_global_flags * gfp UNUSED, size_t n UNUSED)
   { return 0; }
-static size_t id3tag_set_pad_stub(lame_global_flags * gfp UNUSED, size_t n UNUSED)
+UNUSED static size_t lame_get_id3v2_tag_stub(lame_global_flags * gfp UNUSED, unsigned char * buffer UNUSED, size_t size UNUSED)
   { return 0; }
-static size_t lame_get_id3v2_tag_stub(lame_global_flags * gfp UNUSED, unsigned char * buffer UNUSED, size_t size UNUSED)
-  { return 0; }
-static int id3tag_set_fieldvalue_stub(lame_global_flags * gfp UNUSED, const char *fieldvalue UNUSED)
+UNUSED static int id3tag_set_fieldvalue_stub(lame_global_flags * gfp UNUSED, const char *fieldvalue UNUSED)
   { return 0; }
 
 static int get_id3v2_tag_size(sox_format_t * ft)
@@ -601,14 +739,14 @@ static int get_id3v2_tag_size(sox_format_t * ft)
   }
 
   /* read 10 bytes in case there's an ID3 version 2 header here */
-  bytes_read = fread(id3v2_header, 1, sizeof(id3v2_header), fp);
+  bytes_read = fread(id3v2_header, (size_t)1, sizeof(id3v2_header), fp);
   if (bytes_read != sizeof(id3v2_header)) {
     lsx_warn("cannot update id3 tag - failed to read id3 header");
     return SOX_EOF;      /* not readable, maybe opened Write-Only */
   }
 
   /* does the stream begin with the ID3 version 2 file identifier? */
-  if (!strncmp((char *) id3v2_header, "ID3", 3)) {
+  if (!strncmp((char *) id3v2_header, "ID3", (size_t)3)) {
     /* the tag size (minus the 10-byte header) is encoded into four
      * bytes where the most significant bit is clear in each byte */
     id3v2_size = (((id3v2_header[6] & 0x7f) << 21)
@@ -623,7 +761,7 @@ static int get_id3v2_tag_size(sox_format_t * ft)
   return id3v2_size;
 }
 
-static void rewrite_id3v2_tag(sox_format_t * ft, size_t id3v2_size, size_t num_samples)
+static void rewrite_id3v2_tag(sox_format_t * ft, size_t id3v2_size, uint64_t num_samples)
 {
   priv_t *p = (priv_t *)ft->priv;
   FILE *fp = ft->fp;
@@ -646,8 +784,13 @@ static void rewrite_id3v2_tag(sox_format_t * ft, size_t id3v2_size, size_t num_s
     return;
   }
 
-  p->lame_set_num_samples(p->gfp, num_samples);
-  lsx_debug("updated MP3 TLEN to %d samples", num_samples);
+  if (num_samples > ULONG_MAX)
+  {
+    lsx_warn("cannot accurately update track length info - file is too long");
+    num_samples = 0;
+  }
+  p->lame_set_num_samples(p->gfp, (unsigned long)num_samples);
+  lsx_debug("updated MP3 TLEN to %" PRIu64 " samples", num_samples);
 
   new_size = p->lame_get_id3v2_tag(p->gfp, buffer, id3v2_size);
 
@@ -669,15 +812,15 @@ static void rewrite_id3v2_tag(sox_format_t * ft, size_t id3v2_size, size_t num_s
   } else {
     fseeko(fp, (off_t)0, SEEK_SET);
     /* Overwrite the Id3v2 tag (this time TLEN should be accurate) */
-    if (fwrite(buffer, id3v2_size, 1, fp) != 1) {
-      lsx_debug("Rewrote Id3v2 tag (%d bytes)", id3v2_size);
+    if (fwrite(buffer, id3v2_size, (size_t)1, fp) != 1) {
+      lsx_debug("Rewrote Id3v2 tag (%" PRIuPTR " bytes)", id3v2_size);
     }
   }
 
   free(buffer);
 }
 
-static void rewrite_tags(sox_format_t * ft, unsigned long num_samples)
+static void rewrite_tags(sox_format_t * ft, uint64_t num_samples)
 {
   priv_t *p = (priv_t *)ft->priv;
   FILE *fp = ft->fp;
@@ -704,7 +847,7 @@ static void rewrite_tags(sox_format_t * ft, unsigned long num_samples)
   }
 
   if (p->vbr_tag) {
-    unsigned int lametag_size;
+    size_t lametag_size;
     uint8_t buffer[MAXFRAMESIZE];
 
     if (fseeko(fp, (off_t)id3v2_size, SEEK_SET)) {
@@ -722,75 +865,146 @@ static void rewrite_tags(sox_format_t * ft, unsigned long num_samples)
       return;
     }
 
-    if (fwrite(buffer, lametag_size, 1, fp) != 1) {
+    if (fwrite(buffer, lametag_size, (size_t)1, fp) != 1) {
       lsx_warn("cannot write VBR tag - VBR tag write failed");
     } else {
-      lsx_debug("rewrote VBR tag (%u bytes)", lametag_size);
+      lsx_debug("rewrote VBR tag (%" PRIuPTR " bytes)", lametag_size);
     }
   }
 }
 
+#endif /* HAVE_LAME */
+
+#if defined(HAVE_LAME) || defined(HAVE_TWOLAME)
+
 #define LAME_BUFFER_SIZE(num_samples) (((num_samples) + 3) / 4 * 5 + 7200)
 
 static int startwrite(sox_format_t * ft)
 {
   priv_t *p = (priv_t *) ft->priv;
   int openlibrary_result;
-
-  LSX_DLLIBRARY_OPEN(
-      p,
-      lame_dl,
-      LAME_FUNC_ENTRIES,
-      "LAME encoder library",
-      lame_library_names,
-      openlibrary_result);
-  if (openlibrary_result)
-    return SOX_EOF;
+  int fail = 0;
 
   if (ft->encoding.encoding != SOX_ENCODING_MP3) {
     if(ft->encoding.encoding != SOX_ENCODING_UNKNOWN)
-      lsx_report("Encoding forced to MP3");
+      lsx_report("Encoding forced to MP2/MP3");
     ft->encoding.encoding = SOX_ENCODING_MP3;
   }
 
+  if(strchr(ft->filetype, '2'))
+      p->mp2 = 1;
+
+  if (p->mp2) {
+#ifdef HAVE_TWOLAME
+    LSX_DLLIBRARY_OPEN(
+        p,
+        twolame_dl,
+        TWOLAME_FUNC_ENTRIES,
+        "Twolame encoder library",
+        twolame_library_names,
+        openlibrary_result);
+#else
+    lsx_fail_errno(ft,SOX_EOF,"SoX was compiled without MP2 encoding support");
+    return SOX_EOF;
+#endif
+  } else {
+#ifdef HAVE_LAME
+    LSX_DLLIBRARY_OPEN(
+        p,
+        lame_dl,
+        LAME_FUNC_ENTRIES,
+        "LAME encoder library",
+        lame_library_names,
+        openlibrary_result);
+#else
+    lsx_fail_errno(ft,SOX_EOF,"SoX was compiled without MP3 encoding support");
+    return SOX_EOF;
+#endif
+  }
+  if (openlibrary_result)
+    return SOX_EOF;
+
   p->mp3_buffer_size = LAME_BUFFER_SIZE(sox_globals.bufsiz / max(ft->signal.channels, 1));
   p->mp3_buffer = lsx_malloc(p->mp3_buffer_size);
 
   p->pcm_buffer_size = sox_globals.bufsiz * sizeof(float);
   p->pcm_buffer = lsx_malloc(p->pcm_buffer_size);
 
-  p->gfp = p->lame_init();
+  if (p->mp2) {
+#ifdef HAVE_TWOLAME
+    p->opt = p->twolame_init();
 
-  if (p->gfp == NULL){
-    lsx_fail_errno(ft,SOX_EOF,"Initialization of LAME library failed");
-    return(SOX_EOF);
+    if (p->opt == NULL){
+      lsx_fail_errno(ft,SOX_EOF,"Initialization of Twolame library failed");
+      return(SOX_EOF);
+    }
+#endif
+  } else {
+#ifdef HAVE_LAME
+    p->gfp = p->lame_init();
+
+    if (p->gfp == NULL){
+      lsx_fail_errno(ft,SOX_EOF,"Initialization of LAME library failed");
+      return(SOX_EOF);
+    }
+
+    /* First set message callbacks so we don't miss any messages: */
+    p->lame_set_errorf(p->gfp,errorf);
+    p->lame_set_debugf(p->gfp,debugf);
+    p->lame_set_msgf  (p->gfp,msgf);
+
+    p->num_samples = ft->signal.length == SOX_IGNORE_LENGTH ? 0 : ft->signal.length / max(ft->signal.channels, 1);
+    p->lame_set_num_samples(p->gfp, p->num_samples > ULONG_MAX ? 0 : (unsigned long)p->num_samples);
+#endif
   }
 
-  /* First set message callbacks so we don't miss any messages: */
-  p->lame_set_errorf(p->gfp,errorf);
-  p->lame_set_debugf(p->gfp,debugf);
-  p->lame_set_msgf  (p->gfp,msgf);
-
-  p->num_samples = ft->signal.length == SOX_IGNORE_LENGTH ? 0 : ft->signal.length / max(ft->signal.channels, 1);
-  p->lame_set_num_samples(p->gfp, p->num_samples);
-
   ft->signal.precision = MP3_LAME_PRECISION;
 
   if (ft->signal.channels != SOX_ENCODING_UNKNOWN) {
-    if ( (p->lame_set_num_channels(p->gfp,(int)ft->signal.channels)) < 0) {
-        lsx_fail_errno(ft,SOX_EOF,"Unsupported number of channels");
-        return(SOX_EOF);
+    if (p->mp2) {
+#ifdef HAVE_TWOLAME
+      fail = (p->twolame_set_num_channels(p->opt,(int)ft->signal.channels) != 0);
+#endif
+    } else {
+#ifdef HAVE_LAME
+      fail = (p->lame_set_num_channels(p->gfp,(int)ft->signal.channels) < 0);
+#endif
+    }
+    if (fail) {
+      lsx_fail_errno(ft,SOX_EOF,"Unsupported number of channels");
+      return(SOX_EOF);
+    }
+  }
+  else {
+    if (p->mp2) {
+#ifdef HAVE_TWOLAME
+      ft->signal.channels = p->twolame_get_num_channels(p->opt); /* Twolame default */
+#endif
+    } else {
+#ifdef HAVE_LAME
+      ft->signal.channels = p->lame_get_num_channels(p->gfp); /* LAME default */
+#endif
     }
   }
-  else
-    ft->signal.channels = p->lame_get_num_channels(p->gfp); /* LAME default */
 
-  p->lame_set_in_samplerate(p->gfp,(int)ft->signal.rate);
-  p->lame_set_out_samplerate(p->gfp,(int)ft->signal.rate);
+  if (p->mp2) {
+#ifdef HAVE_TWOLAME
+    p->twolame_set_in_samplerate(p->opt,(int)ft->signal.rate);
+    p->twolame_set_out_samplerate(p->opt,(int)ft->signal.rate);
+#endif
+  } else {
+#ifdef HAVE_LAME
+    p->lame_set_in_samplerate(p->gfp,(int)ft->signal.rate);
+    p->lame_set_out_samplerate(p->gfp,(int)ft->signal.rate);
+#endif
+  }
 
-  
-  if (!LSX_DLFUNC_IS_STUB(p, id3tag_init))
-    write_comments(ft);
+  if (!p->mp2) {
+#ifdef HAVE_LAME
+    if (!LSX_DLFUNC_IS_STUB(p, id3tag_init))
+      write_comments(ft);
+#endif
+  }
 
   /* The primary parameter to the LAME encoder is the bit rate. If the
    * value of encoding.compression is a positive integer, it's taken as
@@ -831,7 +1045,7 @@ static int startwrite(sox_format_t * ft)
 
   if (ft->encoding.compression == HUGE_VAL) {
     /* Do nothing, use defaults: */
-    lsx_report("using MP3 encoding defaults");
+    lsx_report("using %s encoding defaults", p->mp2? "MP2" : "MP3");
   } else {
     double abs_compression = fabs(ft->encoding.compression);
     double floor_compression = floor(abs_compression);
@@ -843,15 +1057,16 @@ static int startwrite(sox_format_t * ft)
         : (int)(fraction_compression * 10.0 + 0.5);
 
     if (ft->encoding.compression < 0.5) {
+      if (p->mp2) {
+        lsx_fail_errno(ft,SOX_EOF,"Variable bitrate encoding not supported for MP2 audio");
+        return(SOX_EOF);
+      }
+#ifdef HAVE_LAME
       if (p->lame_get_VBR(p->gfp) == vbr_off)
         p->lame_set_VBR(p->gfp, vbr_default);
 
       if (ft->seekable) {
-        if (!LSX_DLFUNC_IS_STUB(p, lame_get_lametag_frame)) {
-          p->vbr_tag = 1;
-        } else {
-          lsx_report("unable to write VBR tag because lametag update is not supported with this version of LAME");
-        }
+        p->vbr_tag = 1;
       } else {
         lsx_warn("unable to write VBR tag because we can't seek");
       }
@@ -864,35 +1079,60 @@ static int startwrite(sox_format_t * ft)
         return(SOX_EOF);
       }
       lsx_report("lame_set_VBR_q(%d)", bitrate_q);
+#endif
     } else {
-      if (p->lame_set_brate(p->gfp, bitrate_q) < 0) {
+      if (p->mp2) {
+#ifdef HAVE_TWOLAME
+        fail = (p->twolame_set_brate(p->opt, bitrate_q) != 0);
+#endif
+      } else {
+#ifdef HAVE_LAME
+        fail = (p->lame_set_brate(p->gfp, bitrate_q) < 0);
+#endif
+      }
+      if (fail) {
         lsx_fail_errno(ft, SOX_EOF,
-          "lame_set_brate(%d) failed", bitrate_q);
+          "%slame_set_brate(%d) failed", p->mp2? "two" : "", bitrate_q);
         return(SOX_EOF);
       }
-      lsx_report("lame_set_brate(%d)", bitrate_q);
+      lsx_report("(two)lame_set_brate(%d)", bitrate_q);
     }
 
     /* Set Quality */
 
-    if (encoder_q < 0) {
+    if (encoder_q < 0 || p->mp2) {
       /* use default quality value */
-      lsx_report("using MP3 default quality");
+      lsx_report("using %s default quality", p->mp2? "MP2" : "MP3");
     } else {
+#ifdef HAVE_LAME
       if (p->lame_set_quality(p->gfp, encoder_q) < 0) {
         lsx_fail_errno(ft, SOX_EOF,
           "lame_set_quality(%d) failed", encoder_q);
         return(SOX_EOF);
       }
       lsx_report("lame_set_quality(%d)", encoder_q);
+#endif
     }
   }
 
-  p->lame_set_bWriteVbrTag(p->gfp, p->vbr_tag);
+  if (!p->mp2) {
+#ifdef HAVE_LAME
+    p->lame_set_bWriteVbrTag(p->gfp, p->vbr_tag);
+#endif
+  }
 
-  if (p->lame_init_params(p->gfp) < 0){
-        lsx_fail_errno(ft,SOX_EOF,"LAME initialization failed");
-        return(SOX_EOF);
+  if (p->mp2) {
+#ifdef HAVE_TWOLAME
+    fail = (p->twolame_init_params(p->opt) != 0);
+#endif
+  } else {
+#ifdef HAVE_LAME
+    fail = (p->lame_init_params(p->gfp) < 0);
+#endif
+  }
+  if (fail) {
+    lsx_fail_errno(ft,SOX_EOF,"%s initialization failed", p->mp2? "Twolame" : "LAME");
+    return(SOX_EOF);
   }
 
   return(SOX_SUCCESS);
@@ -907,7 +1147,7 @@ static size_t sox_mp3write(sox_format_t * ft, const sox_sample_t *buf, size_t sa
     float *buffer_l, *buffer_r = NULL;
     int nsamples = samp/ft->signal.channels;
     int i,j;
-    size_t written;
+    int written = 0;
     int clips = 0;
     SOX_SAMPLE_LOCALS;
 
@@ -924,24 +1164,33 @@ static size_t sox_mp3write(sox_format_t * ft, const sox_sample_t *buf, size_t sa
 
     buffer_l = p->pcm_buffer;
 
-    if (ft->signal.channels == 2)
+    if (p->mp2)
     {
-        /* lame doesn't support interleaved samples for floats so we must break
-         * them out into seperate buffers.
-         */
-        buffer_r = p->pcm_buffer + nsamples;
-        j=0;
-        for (i = 0; i < nsamples; i++)
-        {
-            buffer_l[i] = MP3_SAMPLE_TO_FLOAT(buf[j++], clips);
-            buffer_r[i] = MP3_SAMPLE_TO_FLOAT(buf[j++], clips);
-        }
+        size_t s;
+        for(s = 0; s < samp; s++)
+            buffer_l[s] = SOX_SAMPLE_TO_FLOAT_32BIT(buf[s], clips);
     }
     else
     {
-        j=0;
-        for (i = 0; i < nsamples; i++) {
-            buffer_l[i] = MP3_SAMPLE_TO_FLOAT(buf[j++], clips);
+        if (ft->signal.channels == 2)
+        {
+            /* lame doesn't support interleaved samples for floats so we must break
+             * them out into seperate buffers.
+             */
+            buffer_r = p->pcm_buffer + nsamples;
+            j=0;
+            for (i = 0; i < nsamples; i++)
+            {
+                buffer_l[i] = MP3_SAMPLE_TO_FLOAT(buf[j++], clips);
+                buffer_r[i] = MP3_SAMPLE_TO_FLOAT(buf[j++], clips);
+            }
+        }
+        else
+        {
+            j=0;
+            for (i = 0; i < nsamples; i++) {
+                buffer_l[i] = MP3_SAMPLE_TO_FLOAT(buf[j++], clips);
+            }
         }
     }
 
@@ -956,15 +1205,23 @@ static size_t sox_mp3write(sox_format_t * ft, const sox_sample_t *buf, size_t sa
       p->mp3_buffer = new_buffer;
     }
 
-    if ((written =
-      p->lame_encode_buffer_float(p->gfp, buffer_l, buffer_r,
-                   nsamples, p->mp3_buffer,
-                   (int)p->mp3_buffer_size)) > p->mp3_buffer_size){
+    if(p->mp2) {
+#ifdef HAVE_TWOLAME
+        written = p->twolame_encode_buffer_float32_interleaved(p->opt, buffer_l,
+                  nsamples, p->mp3_buffer, (int)p->mp3_buffer_size);
+#endif
+    } else {
+#ifdef HAVE_LAME
+        written = p->lame_encode_buffer_float(p->gfp, buffer_l, buffer_r,
+                  nsamples, p->mp3_buffer, (int)p->mp3_buffer_size);
+#endif
+    }
+    if (written < 0) {
         lsx_fail_errno(ft,SOX_EOF,"Encoding failed");
         return 0;
     }
 
-    if (lsx_writebuf(ft, p->mp3_buffer, written) < written)
+    if (lsx_writebuf(ft, p->mp3_buffer, (size_t)written) < (size_t)written)
     {
         lsx_fail_errno(ft,SOX_EOF,"File write failed");
         return 0;
@@ -976,44 +1233,69 @@ static size_t sox_mp3write(sox_format_t * ft, const sox_sample_t *buf, size_t sa
 static int stopwrite(sox_format_t * ft)
 {
   priv_t *p = (priv_t *) ft->priv;
-  unsigned long num_samples = ft->olength == SOX_IGNORE_LENGTH ? 0 : ft->olength / max(ft->signal.channels, 1);
-  int written = p->lame_encode_flush(p->gfp, p->mp3_buffer, (int)p->mp3_buffer_size);
+  uint64_t num_samples = ft->olength == SOX_IGNORE_LENGTH ? 0 : ft->olength / max(ft->signal.channels, 1);
+  int written = 0;
 
+  if (p->mp2) {
+#ifdef HAVE_TWOLAME
+    written = p->twolame_encode_flush(p->opt, p->mp3_buffer, (int)p->mp3_buffer_size);
+#endif
+  } else {
+#ifdef HAVE_LAME
+    written = p->lame_encode_flush(p->gfp, p->mp3_buffer, (int)p->mp3_buffer_size);
+#endif
+  }
   if (written < 0)
     lsx_fail_errno(ft, SOX_EOF, "Encoding failed");
   else if (lsx_writebuf(ft, p->mp3_buffer, (size_t)written) < (size_t)written)
     lsx_fail_errno(ft, SOX_EOF, "File write failed");
-  else if (ft->seekable && (num_samples != p->num_samples || p->vbr_tag))
-    rewrite_tags(ft, num_samples);
+  else if (!p->mp2) {
+#ifdef HAVE_LAME
+    if (ft->seekable && (num_samples != p->num_samples || p->vbr_tag))
+      rewrite_tags(ft, num_samples);
+#endif
+  }
 
   free(p->mp3_buffer);
   free(p->pcm_buffer);
-  p->lame_close(p->gfp);
-  LSX_DLLIBRARY_CLOSE(p, lame_dl);
+
+  if(p->mp2) {
+#ifdef HAVE_TWOLAME
+    p->twolame_close(&p->opt);
+    LSX_DLLIBRARY_CLOSE(p, twolame_dl);
+#endif
+  } else {
+#ifdef HAVE_LAME
+    p->lame_close(p->gfp);
+    LSX_DLLIBRARY_CLOSE(p, lame_dl);
+#endif
+  }
   return SOX_SUCCESS;
 }
 
-#else /* HAVE_LAME */
+#else /* !(HAVE_LAME || HAVE_TWOLAME) */
 static int startwrite(sox_format_t * ft UNUSED)
 {
-  lsx_fail_errno(ft,SOX_EOF,"SoX was compiled without MP3 encoding support");
+  lsx_fail_errno(ft,SOX_EOF,"SoX was compiled with neither MP2 nor MP3 encoding support");
   return SOX_EOF;
 }
 #define sox_mp3write NULL
 #define stopwrite NULL
-#endif /* HAVE_LAME */
+#endif /* HAVE_LAME || HAVE_TWOLAME */
 
 LSX_FORMAT_HANDLER(mp3)
 {
   static char const * const names[] = {"mp3", "mp2", "audio/mpeg", NULL};
   static unsigned const write_encodings[] = {
     SOX_ENCODING_MP3, 0, 0};
+  static sox_rate_t const write_rates[] = {
+    8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000, 0};
   static sox_format_handler_t const handler = {SOX_LIB_VERSION_CODE,
-    "MPEG Layer 3 lossy audio compression", names, 0,
+    "MPEG Layer 2/3 lossy audio compression", names, 0,
     startread, sox_mp3read, stopread,
     startwrite, sox_mp3write, stopwrite,
-    NULL, write_encodings, NULL, sizeof(priv_t)
+    sox_mp3seek, write_encodings, write_rates, sizeof(priv_t)
   };
   return &handler;
 }
-#endif /* defined(HAVE_MAD_H) || defined(HAVE_LAME) */
+#endif /* defined(HAVE_MAD_H) || defined(HAVE_LAME) || defined(HAVE_TWOLAME) */
