@@ -36,6 +36,17 @@
   #include <machine/soundcard.h>
 #endif
 
+/* these appear in the sys/soundcard.h of OSS 4.x, and in Linux's
+ * sound/core/oss/pcm_oss.c (2.6.24 and later), but are typically
+ * not included in system header files.
+ */
+#ifndef AFMT_S32_LE
+#define AFMT_S32_LE 0x00001000
+#endif
+#ifndef AFMT_S32_BE
+#define AFMT_S32_BE 0x00002000
+#endif
+
 #include <sys/ioctl.h>
 
 typedef sox_fileinfo_t priv_t;
@@ -45,9 +56,8 @@ static int ossinit(sox_format_t * ft)
     int sampletype, samplesize, dsp_stereo;
     int tmp, rc;
     priv_t *file = (priv_t *)ft->priv;
-    sox_signalinfo_t client_signal = ft->signal;
+    int fn = fileno((FILE *)(ft->fp));
 
-    lsx_set_signal_defaults(&ft->signal);
     if (ft->encoding.bits_per_sample == 8) {
         sampletype = AFMT_U8;
         samplesize = 8;
@@ -74,6 +84,21 @@ static int ossinit(sox_format_t * ft)
             ft->encoding.encoding = SOX_ENCODING_SIGN2;
         }
     }
+    else if (ft->encoding.bits_per_sample == 32) {
+	/* Attempt to use endian that user specified */
+	if (ft->encoding.reverse_bytes)
+	    sampletype = (MACHINE_IS_BIGENDIAN) ? AFMT_S32_LE : AFMT_S32_BE;
+	else
+	    sampletype = (MACHINE_IS_BIGENDIAN) ? AFMT_S32_BE : AFMT_S32_LE;
+	samplesize = 32;
+	if (ft->encoding.encoding == SOX_ENCODING_UNKNOWN)
+	    ft->encoding.encoding = SOX_ENCODING_SIGN2;
+	if (ft->encoding.encoding != SOX_ENCODING_SIGN2) {
+	    lsx_report("OSS driver only supports signed with words");
+	    lsx_report("Forcing to signed linear");
+	    ft->encoding.encoding = SOX_ENCODING_SIGN2;
+	}
+    }
     else {
         /* Attempt to use endian that user specified */
         if (ft->encoding.reverse_bytes)
@@ -89,7 +114,7 @@ static int ossinit(sox_format_t * ft)
 
     if (ft->signal.channels > 2) ft->signal.channels = 2;
 
-    if (ioctl(fileno(ft->fp), (size_t) SNDCTL_DSP_RESET, 0) < 0)
+    if (ioctl(fn, (size_t) SNDCTL_DSP_RESET, 0) < 0)
     {
         lsx_fail_errno(ft,SOX_EOF,"Unable to reset OSS driver.  Possibly accessing an invalid file/device");
         return(SOX_EOF);
@@ -97,7 +122,7 @@ static int ossinit(sox_format_t * ft)
 
     /* Query the supported formats and find the best match
      */
-    rc = ioctl(fileno(ft->fp), SNDCTL_DSP_GETFMTS, &tmp);
+    rc = ioctl(fn, SNDCTL_DSP_GETFMTS, &tmp);
     if (rc == 0) {
         if ((tmp & sampletype) == 0)
         {
@@ -136,7 +161,7 @@ static int ossinit(sox_format_t * ft)
 
         }
         tmp = sampletype;
-        rc = ioctl(fileno(ft->fp), SNDCTL_DSP_SETFMT, &tmp);
+        rc = ioctl(fn, SNDCTL_DSP_SETFMT, &tmp);
     }
     /* Give up and exit */
     if (rc < 0 || tmp != sampletype)
@@ -149,21 +174,17 @@ static int ossinit(sox_format_t * ft)
     else dsp_stereo = 0;
 
     tmp = dsp_stereo;
-    if (ioctl(fileno(ft->fp), SNDCTL_DSP_STEREO, &tmp) < 0)
+    if (ioctl(fn, SNDCTL_DSP_STEREO, &tmp) < 0)
     {
         lsx_warn("Couldn't set to %s", dsp_stereo?  "stereo":"mono");
         dsp_stereo = 0;
     }
 
     if (tmp != dsp_stereo)
-    {
-      if (client_signal.channels != 0)
-        lsx_warn("Sound card appears to support only %d channels.  Overriding format", tmp+1);
         ft->signal.channels = tmp + 1;
-    }
 
     tmp = ft->signal.rate;
-    if (ioctl (fileno(ft->fp), SNDCTL_DSP_SPEED, &tmp) < 0 ||
+    if (ioctl(fn, SNDCTL_DSP_SPEED, &tmp) < 0 ||
         (int)ft->signal.rate != tmp) {
         /* If the rate the sound card is using is not within 1% of what
          * the user specified then override the user setting.
@@ -174,28 +195,24 @@ static int ossinit(sox_format_t * ft)
          * we can't hear anyways.
          */
         if ((int)ft->signal.rate - tmp > (tmp * .01) ||
-            tmp - (int)ft->signal.rate > (tmp * .01)) {
-          if (client_signal.rate != 0)
-            lsx_warn("Unable to set audio speed to %g (set to %d)",
-                     ft->signal.rate, tmp);
+            tmp - (int)ft->signal.rate > (tmp * .01))
             ft->signal.rate = tmp;
-        }
     }
 
     /* Find out block size to use last because the driver could compute
      * its size based on specific rates/formats.
      */
     file->size = 0;
-    ioctl (fileno(ft->fp), SNDCTL_DSP_GETBLKSIZE, &file->size);
+    ioctl(fn, SNDCTL_DSP_GETBLKSIZE, &file->size);
     if (file->size < 4 || file->size > 65536) {
-            lsx_fail_errno(ft,SOX_EOF,"Invalid audio buffer size %lu", (unsigned long)file->size);
+            lsx_fail_errno(ft,SOX_EOF,"Invalid audio buffer size %" PRIuPTR, file->size);
             return (SOX_EOF);
     }
     file->count = 0;
     file->pos = 0;
     file->buf = lsx_malloc(file->size);
 
-    if (ioctl(fileno(ft->fp), (size_t) SNDCTL_DSP_SYNC, NULL) < 0) {
+    if (ioctl(fn, (size_t) SNDCTL_DSP_SYNC, NULL) < 0) {
         lsx_fail_errno(ft,SOX_EOF,"Unable to sync dsp");
         return (SOX_EOF);
     }
@@ -205,11 +222,11 @@ static int ossinit(sox_format_t * ft)
     return(SOX_SUCCESS);
 }
 
-SOX_FORMAT_HANDLER(oss)
+LSX_FORMAT_HANDLER(oss)
 {
   static char const * const names[] = {"ossdsp", "oss", NULL};
   static unsigned const write_encodings[] = {
-    SOX_ENCODING_SIGN2, 16, 0,
+    SOX_ENCODING_SIGN2, 32, 16, 0,
     SOX_ENCODING_UNSIGNED, 8, 0,
     0};
   static sox_format_handler_t const handler = {SOX_LIB_VERSION_CODE,
