@@ -13,24 +13,28 @@
 #ifndef SOX_I_H
 #define SOX_I_H
 
-#include "soxomp.h"  /* Make this 1st in list (for soxconfig) */
+#include "soxomp.h"  /* Note: soxomp.h includes soxconfig.h */
+#include "sox.h"
+
 #if defined HAVE_FMEMOPEN
 #define _GNU_SOURCE
 #endif
-#include "sox.h"
-#include "util.h"
 
 #include <errno.h>
+#include <stdio.h>
+#include <stdlib.h>
+
+#include "util.h"
 
 #if defined(LSX_EFF_ALIAS)
 #undef lsx_debug
 #undef lsx_fail
 #undef lsx_report
 #undef lsx_warn
-#define lsx_debug sox_globals.subsystem=effp->handler.name,lsx_debug
-#define lsx_fail sox_globals.subsystem=effp->handler.name,lsx_fail
-#define lsx_report sox_globals.subsystem=effp->handler.name,lsx_report
-#define lsx_warn sox_globals.subsystem=effp->handler.name,lsx_warn
+#define lsx_debug sox_globals.subsystem=effp->handler.name,lsx_debug_impl
+#define lsx_fail sox_globals.subsystem=effp->handler.name,lsx_fail_impl
+#define lsx_report sox_globals.subsystem=effp->handler.name,lsx_report_impl
+#define lsx_warn sox_globals.subsystem=effp->handler.name,lsx_warn_impl
 #endif
 
 #define RANQD1 ranqd1(sox_globals.ranqd1)
@@ -38,7 +42,7 @@
 
 typedef enum {SOX_SHORT, SOX_INT, SOX_FLOAT, SOX_DOUBLE} sox_data_t;
 typedef enum {SOX_WAVE_SINE, SOX_WAVE_TRIANGLE} lsx_wave_t;
-extern lsx_enum_item const lsx_wave_enum[];
+lsx_enum_item const * lsx_get_wave_enum(void);
 
 /* Define fseeko and ftello for platforms lacking them */
 #ifndef HAVE_FSEEKO
@@ -50,21 +54,13 @@ extern lsx_enum_item const lsx_wave_enum[];
 assert_static(sizeof(off_t) == _FILE_OFFSET_BITS >> 3, OFF_T_BUILD_PROBLEM);
 #endif
 
-#if defined __GNUC__
-#define FMT_size_t "zu"
-#elif defined _MSC_VER
-#define FMT_size_t "Iu"
-#else
-#define FMT_size_t "lu"
-#endif
-
 FILE * lsx_tmpfile(void);
 
-void lsx_debug_more(char const * fmt, ...) PRINTF;
-void lsx_debug_most(char const * fmt, ...) PRINTF;
+void lsx_debug_more_impl(char const * fmt, ...) LSX_PRINTF12;
+void lsx_debug_most_impl(char const * fmt, ...) LSX_PRINTF12;
 
-#define lsx_debug_more sox_globals.subsystem=__FILE__,lsx_debug_more
-#define lsx_debug_most sox_globals.subsystem=__FILE__,lsx_debug_most
+#define lsx_debug_more sox_get_globals()->subsystem=__FILE__,lsx_debug_more_impl
+#define lsx_debug_most sox_get_globals()->subsystem=__FILE__,lsx_debug_most_impl
 
 /* Digitise one cycle of a wave and store it as
  * a table of samples of a specified data-type.
@@ -77,7 +73,7 @@ void lsx_generate_wave_table(
     double min,         /* Minimum value on the y-axis. (e.g. -1) */
     double max,         /* Maximum value on the y-axis. (e.g. +1) */
     double phase);      /* Phase at 1st point; 0..2pi. (e.g. pi/2 for cosine) */
-char const * lsx_parsesamples(sox_rate_t rate, const char *str, size_t *samples, int def);
+char const * lsx_parsesamples(sox_rate_t rate, const char *str, uint64_t *samples, int def);
 int lsx_parse_note(char const * text, char * * end_ptr);
 double lsx_parse_frequency_k(char const * text, char * * end_ptr, int key);
 #define lsx_parse_frequency(a, b) lsx_parse_frequency_k(a, b, INT_MAX)
@@ -128,6 +124,9 @@ void lsx_plot_fir(double * h, int num_points, sox_rate_t rate, sox_plot_t type,
 #include <byteswap.h>
 #define lsx_swapw(x) bswap_16(x)
 #define lsx_swapdw(x) bswap_32(x)
+#elif defined(_MSC_VER)
+#define lsx_swapw(x) _byteswap_ushort(x)
+#define lsx_swapdw(x) _byteswap_ulong(x)
 #else
 #define lsx_swapw(uw) (((uw >> 8) | (uw << 8)) & 0xffff)
 #define lsx_swapdw(udw) ((udw >> 24) | ((udw >> 8) & 0xff00) | ((udw << 8) & 0xff0000) | (udw << 24))
@@ -147,7 +146,7 @@ int lsx_writes(sox_format_t * ft, char const * c);
 void lsx_set_signal_defaults(sox_format_t * ft);
 #define lsx_writechars(ft, chars, len) (lsx_writebuf(ft, chars, len) == len? SOX_SUCCESS : SOX_EOF)
 
-size_t lsx_read_3_buf(sox_format_t * ft, uint24_t *buf, size_t len);
+size_t lsx_read_3_buf(sox_format_t * ft, sox_uint24_t *buf, size_t len);
 size_t lsx_read_b_buf(sox_format_t * ft, uint8_t *buf, size_t len);
 size_t lsx_read_df_buf(sox_format_t * ft, double *buf, size_t len);
 size_t lsx_read_dw_buf(sox_format_t * ft, uint32_t *buf, size_t len);
@@ -155,7 +154,7 @@ size_t lsx_read_qw_buf(sox_format_t * ft, uint64_t *buf, size_t len);
 size_t lsx_read_f_buf(sox_format_t * ft, float *buf, size_t len);
 size_t lsx_read_w_buf(sox_format_t * ft, uint16_t *buf, size_t len);
 
-size_t lsx_write_3_buf(sox_format_t * ft, uint24_t *buf, size_t len);
+size_t lsx_write_3_buf(sox_format_t * ft, sox_uint24_t *buf, size_t len);
 size_t lsx_write_b_buf(sox_format_t * ft, uint8_t *buf, size_t len);
 size_t lsx_write_df_buf(sox_format_t * ft, double *buf, size_t len);
 size_t lsx_write_dw_buf(sox_format_t * ft, uint32_t *buf, size_t len);
@@ -163,7 +162,7 @@ size_t lsx_write_qw_buf(sox_format_t * ft, uint64_t *buf, size_t len);
 size_t lsx_write_f_buf(sox_format_t * ft, float *buf, size_t len);
 size_t lsx_write_w_buf(sox_format_t * ft, uint16_t *buf, size_t len);
 
-int lsx_read3(sox_format_t * ft, uint24_t * u3);
+int lsx_read3(sox_format_t * ft, sox_uint24_t * u3);
 int lsx_readb(sox_format_t * ft, uint8_t * ub);
 int lsx_readchars(sox_format_t * ft, char * chars, size_t len);
 int lsx_readdf(sox_format_t * ft, double * d);
@@ -198,7 +197,7 @@ int lsx_error(sox_format_t * ft);
 int lsx_flush(sox_format_t * ft);
 int lsx_seeki(sox_format_t * ft, off_t offset, int whence);
 int lsx_unreadb(sox_format_t * ft, unsigned ub);
-size_t lsx_filelength(sox_format_t * ft);
+uint64_t lsx_filelength(sox_format_t * ft);
 off_t lsx_tell(sox_format_t * ft);
 void lsx_clearerr(sox_format_t * ft);
 void lsx_rewind(sox_format_t * ft);
@@ -222,11 +221,11 @@ typedef struct sox_formats_globals { /* Global parameters (for formats) */
 
 int lsx_check_read_params(sox_format_t * ft, unsigned channels,
     sox_rate_t rate, sox_encoding_t encoding, unsigned bits_per_sample,
-    off_t num_samples, sox_bool check_length);
+    uint64_t num_samples, sox_bool check_length);
 #define LSX_FORMAT_HANDLER(name) \
 sox_format_handler_t const * lsx_##name##_format_fn(void); \
 sox_format_handler_t const * lsx_##name##_format_fn(void)
-#define div_bits(size, bits) (off_t)((double)(size) * 8 / bits)
+#define div_bits(size, bits) ((uint64_t)(size) * 8 / bits)
 
 /* Raw I/O */
 int lsx_rawstartread(sox_format_t * ft);
@@ -235,7 +234,7 @@ int lsx_rawstopread(sox_format_t * ft);
 int lsx_rawstartwrite(sox_format_t * ft);
 size_t lsx_rawwrite(sox_format_t * ft, const sox_sample_t *buf, size_t nsamp);
 int lsx_rawseek(sox_format_t * ft, uint64_t offset);
-int lsx_rawstart(sox_format_t * ft, sox_bool default_rate, sox_bool default_channels, sox_bool default_length, sox_encoding_t encoding, unsigned size);
+int lsx_rawstart(sox_format_t * ft, sox_bool default_rate, sox_bool default_channels, sox_bool default_length, sox_encoding_t encoding, unsigned bits_per_sample);
 #define lsx_rawstartread(ft) lsx_rawstart(ft, sox_false, sox_false, sox_false, SOX_ENCODING_UNKNOWN, 0)
 #define lsx_rawstartwrite lsx_rawstartread
 #define lsx_rawstopread NULL
@@ -280,10 +279,10 @@ char * lsx_usage_lines(char * * usage, char const * const * lines, size_t n);
   } \
 }
 
-#define GETOPT_NUMERIC(ch, name, min, max) case ch:{ \
+#define GETOPT_NUMERIC(state, ch, name, min, max) case ch:{ \
   char * end_ptr; \
-  double d = strtod(lsx_optarg, &end_ptr); \
-  if (end_ptr == lsx_optarg || d < min || d > max || *end_ptr != '\0') {\
+  double d = strtod(state.arg, &end_ptr); \
+  if (end_ptr == state.arg || d < min || d > max || *end_ptr != '\0') {\
     lsx_fail("parameter `%s' must be between %g and %g", #name, (double)min, (double)max); \
     return lsx_usage(effp); \
   } \
