@@ -28,12 +28,13 @@ typedef struct {
   double factor;
 } priv_t;
 
-static int getopts(sox_effect_t * effp, int n, char * * argv)
+static int getopts(sox_effect_t * effp, int argc, char * * argv)
 {
   priv_t * p = (priv_t *) effp->priv;
   sox_bool is_cents = sox_false;
 
-  if (n == 1) {
+  --argc, ++argv;
+  if (argc == 1) {
     char c, dummy;
     int scanned = sscanf(*argv, "%lf%c %c", &p->factor, &c, &dummy);
     if (scanned == 1 || (scanned == 2 && c == 'c')) {
@@ -55,22 +56,18 @@ static int start(sox_effect_t * effp)
     return SOX_EFF_NULL;
 
   effp->out_signal.rate = effp->in_signal.rate * p->factor;
+
+  effp->out_signal.length = effp->in_signal.length;
+    /* audio length if measured in samples doesn't change */
+
   return SOX_SUCCESS;
 }
 
-static int flow(sox_effect_t * effp, const sox_sample_t * ibuf,
-                sox_sample_t * obuf, size_t * isamp, size_t * osamp)
-{
-  size_t len = *isamp = *osamp = min(*isamp, *osamp);
-  memcpy(obuf, ibuf, len * sizeof(*osamp));
-  (void)effp;
-  return SOX_SUCCESS;
-}
-
-sox_effect_handler_t const * sox_speed_effect_fn(void)
+sox_effect_handler_t const * lsx_speed_effect_fn(void)
 {
   static sox_effect_handler_t handler = {
-    "speed", "factor[c]",SOX_EFF_NULL|SOX_EFF_MCHAN|SOX_EFF_RATE|SOX_EFF_LENGTH,
-    getopts, start, flow, 0, 0, 0, sizeof(priv_t)};
+    "speed", "factor[c]",
+    SOX_EFF_MCHAN | SOX_EFF_RATE | SOX_EFF_LENGTH | SOX_EFF_MODIFY,
+    getopts, start, lsx_flow_copy, 0, 0, 0, sizeof(priv_t)};
   return &handler;
 }
