@@ -623,7 +623,7 @@ static int startread(sox_format_t * ft)
     wav->packet = NULL;
     wav->samples = NULL;
 
-    /* non-PCM formats expect alaw and mulaw formats have extended fmt chunk.
+    /* non-PCM formats except alaw and mulaw formats have extended fmt chunk.
      * Check for those cases.
      */
     if (wav->formatTag != WAVE_FORMAT_PCM &&
@@ -633,7 +633,7 @@ static int startread(sox_format_t * ft)
             lsx_readw(ft, &wExtSize);
             len -= 2;
         } else {
-            lsx_warn("wave header missing FmtExt chunk");
+            lsx_warn("wave header missing extended part of fmt chunk");
         }
     }
 
@@ -782,6 +782,11 @@ static int startread(sox_format_t * ft)
         ft->encoding.encoding = SOX_ENCODING_SIGN2;
       break;
 
+    case 8:
+      if (ft->encoding.encoding == SOX_ENCODING_UNKNOWN)
+        ft->encoding.encoding = SOX_ENCODING_FLOAT;
+      break;
+
     default:
       lsx_fail_errno(ft,SOX_EFMT,"Sorry, don't understand .wav size");
       return SOX_EOF;
@@ -885,7 +890,7 @@ static int startread(sox_format_t * ft)
     }
 
     /* Horrible way to find Cool Edit marker points. Taken from Quake source*/
-    ft->oob.loops[0].start = ~0u;
+    ft->oob.loops[0].start = SOX_IGNORE_LENGTH;
     if(ft->seekable){
         /*Got this from the quake source.  I think it 32bit aligns the chunks
          * doubt any machine writing Cool Edit Chunks writes them at an odd
@@ -1193,6 +1198,7 @@ PCM formats then go straight to the data chunk:
 36 - 39    'data'
 40 - 43     dwDataLength   length of data chunk minus 8 byte header
 44 - (dwDataLength + 43)   the data
+(+ a padding byte if dwDataLength is odd)
 
 non-PCM formats must write an extended format chunk and a fact chunk:
 
@@ -1204,6 +1210,7 @@ ULAW, ALAW formats:
 50 - 53    'data'
 54 - 57     dwDataLength  length of data chunk minus 8 byte header
 58 - (dwDataLength + 57)  the data
+(+ a padding byte if dwDataLength is odd)
 
 
 GSM6.10  format:
@@ -1214,8 +1221,8 @@ GSM6.10  format:
 48 - 51    dwSamplesWritten   actual number of samples written out
 52 - 55    'data'
 56 - 59     dwDataLength  length of data chunk minus 8 byte header
-60 - (dwDataLength + 59)  the data
-(+ a padding byte if dwDataLength is odd)
+60 - (dwDataLength + 59)  the data (including a padding byte, if necessary,
+                            so dwDataLength is always even)
 
 
 note that header contains (up to) 3 separate ways of describing the
@@ -1364,7 +1371,7 @@ static int wavwritehdr(sox_format_t * ft, int second_header)
     else if (wFormatTag != WAVE_FORMAT_PCM)
         wFmtSize += 2+wExtSize; /* plus ExtData */
 
-    wRiffLength = 4 + (8+wFmtSize) + (8+dwDataLength);
+    wRiffLength = 4 + (8+wFmtSize) + (8+dwDataLength+dwDataLength%2);
     if (isExtensible || wFormatTag != WAVE_FORMAT_PCM) /* PCM omits the "fact" chunk */
         wRiffLength += (8+dwFactSize);
 
@@ -1534,6 +1541,12 @@ static int stopwrite(sox_format_t * ft)
             wavgsmstopwrite(ft);
             break;
         }
+
+        /* Add a pad byte if the number of data bytes is odd.
+           See wavwritehdr() above for the calculation. */
+        if (wav->formatTag != WAVE_FORMAT_GSM610)
+          lsx_padbytes(ft, (size_t)((wav->numSamples + wav->samplesPerBlock - 1)/wav->samplesPerBlock*wav->blockAlign) % 2);
+
         free(wav->packet);
         free(wav->samples);
         free(wav->lsx_ms_adpcm_i_coefs);
@@ -1615,7 +1628,7 @@ static int seek(sox_format_t * ft, uint64_t offset)
   if (ft->encoding.bits_per_sample & 7)
     lsx_fail_errno(ft, SOX_ENOTSUP, "seeking not supported with this encoding");
   else if (wav->formatTag == WAVE_FORMAT_GSM610) {
-    int new_offset, alignment;
+    int alignment;
     size_t gsmoff;
 
     /* rounding bytes to blockAlign so that we
@@ -1627,7 +1640,7 @@ static int seek(sox_format_t * ft, uint64_t offset)
     ft->sox_errno = lsx_seeki(ft, (off_t)(gsmoff + wav->dataStart), SEEK_SET);
     if (ft->sox_errno == SOX_SUCCESS) {
       /* offset is in samples */
-      new_offset = offset;
+      uint64_t new_offset = offset;
       alignment = offset % wav->samplesPerBlock;
       if (alignment != 0)
           new_offset += (wav->samplesPerBlock - alignment);
